package org.openoffice.test.jarindex;

import java.util.logging.Logger;

import com.sun.star.frame.DispatchDescriptor;
import com.sun.star.frame.XDispatch;
import com.sun.star.frame.XDispatchProvider;
import com.sun.star.frame.XFrame;
import com.sun.star.lang.XInitialization;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

/**
 * This class implements the component. At least the interfaces XServiceInfo,
 * XTypeProvider, and XInitialization should be provided by the service.
 */
public class ProtocolHandler extends WeakBase implements XDispatchProvider, XInitialization, XServiceInfo {

	private static final Logger logger = Logger.getLogger(ProtocolHandler.class.getName());

	/**
	 * The service name, that must be used to get an instance of this service.
	 */
	private static final String[] SERVICE_NAMES = { "com.sun.star.frame.ProtocolHandler" };

	private static final String PROTOCOL = "org.openoffice.test.jarindex:";

	private static final String COMMAND_TEST = "Test";

	/**
	 * The component context, that gives access to the service manager and all
	 * registered services.
	 */
	private XComponentContext context;

	/**
	 * The frame where the addon depends on.
	 */
	private XFrame frame;

	/**
	 * The constructor of the inner class has a XMultiServiceFactory parameter.
	 * 
	 * @param xmultiservicefactoryInitialization
	 *            A special service factory could be introduced while
	 *            initializing.
	 */
	public ProtocolHandler(XComponentContext xComponentContext) {
		context = xComponentContext;
	}

	/**
	 * This method is a member of the interface for initializing an object
	 * directly after its creation.
	 * 
	 * @param object
	 *            This array of arbitrary objects will be passed to the
	 *            component after its creation.
	 * @throws Exception
	 *             Every exception will not be handled, but will be passed to
	 *             the caller.
	 */
	public void initialize(Object[] object) throws com.sun.star.uno.Exception {
		logger.info("initialize ProtocolHandler");
		if (object.length > 0) {
			frame = (XFrame) UnoRuntime.queryInterface(XFrame.class, object[0]);
		}

	}

	/**
	 * This method returns an array of all supported service names.
	 * 
	 * @return Array of supported service names.
	 */
	public String[] getSupportedServiceNames() {
		return getServiceNames();
	}

	public static String[] getServiceNames() {
		return SERVICE_NAMES;
	}

	/**
	 * This method returns true, if the given service will be supported by the
	 * component.
	 * 
	 * @param stringService
	 *            Service name.
	 * @return True, if the given service name will be supported.
	 */
	public boolean supportsService(String sService) {
		int len = SERVICE_NAMES.length;

		for (int i = 0; i < len; i++) {
			if (sService.equals(SERVICE_NAMES[i]))
				return true;
		}

		return false;
	}

	/**
	 * Return the class name of the component.
	 * 
	 * @return Class name of the component.
	 */
	public String getImplementationName() {
		return ProtocolHandler.class.getName();
	}

	// XDispatchProvider
	public XDispatch queryDispatch(com.sun.star.util.URL aURL, String sTargetFrameName, int iSearchFlags) {
		logger.info("query dispatch [url: " + aURL.Complete + "]");
		XDispatch xRet = null;
		if (PROTOCOL.equals(aURL.Protocol)) {
			if (COMMAND_TEST.equals(aURL.Path))
				xRet = new TestAction(context, frame);
		}
		return xRet;
	}

	public XDispatch[] queryDispatches(DispatchDescriptor[] seqDescripts) {
		int nCount = seqDescripts.length;
		XDispatch[] lDispatcher = new XDispatch[nCount];

		for (int i = 0; i < nCount; ++i)
			lDispatcher[i] = queryDispatch(seqDescripts[i].FeatureURL, seqDescripts[i].FrameName,
					seqDescripts[i].SearchFlags);

		return lDispatcher;
	}
}